/** @file
 * This is the raspberry pi-specific gp3 file. Compile and link with
 * gp3lib.c. Do not link with other files (such as gp3cygwin.c). 
 * @author Al Williams alw@al-williams.com
 * @version 1.1
 * @date 20 Feb 2011
*/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <linux/serial.h>
#include "gp3lib.h"

#include <asm/termios.h>

#define BAUDRATE B57600
#define DEBUGOUT 0
#if DEBUGOUT
#define DEBUG(tag,x) printf("DEBUG: %s=%02X\n",tag,x)
#else
#define DEBUG(tag,x)
#endif

/* This is the part that is platform specific */
/* This is for Raspberry Pi and similar */

static int fd;  // file descriptor
static struct termios oldtio; /* terminal info */


/* Open port at 28800 baud */
int gp3openport(char *port)
{
  struct termios tio;
  int i;

  fd=open(port,O_RDWR|O_NOCTTY);
  if (fd<0)
    {
      return -1;
    }


  tcgetattr(fd,&oldtio); /* save old settings */
  memset(&tio,0,sizeof(tio));
  // Set up for 8N1, no processing 
  tio.c_cflag=BAUDRATE|CS8|CLOCAL|CREAD|CRTSCTS; 
  tio.c_iflag=IGNPAR;
  tio.c_oflag=0;
  tio.c_lflag=0;
  for (i=0;i<NCCS;i++) tio.c_cc[i]=0;
  tio.c_cc[VMIN]=1;
  tcflush(fd,TCIFLUSH);
  tcsetattr(fd,TCSANOW,&tio); // make settings take  

  return 0;
}

/* Close the port when done */
void gp3closeport()
{
  tcsetattr(fd,TCSANOW,&oldtio);
  close(fd);
}

/* Write a byte to the GP3 */
int gp3write(int byte)
{
  char c=(char)byte;
  DEBUG("write",byte);
  return write(fd,&c,1);
}

/* Read a byte from the GP3 */
/* User may call this after repeat */
unsigned int gp3read(void)
{
  char c;
  unsigned int rv;
  read(fd,&c,1);
  rv=((unsigned int)c) & 0xFF;;  
  DEBUG("read",rv);
  return rv;
}

/* Check CTS (optional if you don't want to know if GP3 is busy.
   As written, the system will handle this automatically although
   your program will block until the GP3 is available. This will
   allow you to prevent that if you implement it.
   None of the portable code uses this!

   However, if you don't implement this, you'll need to remove
   the lines that call gp3ready from the example program.

   Not that your Linux/Unix or tty driver may not implement TIOCMGET
   or may not return correct results for this call. 
*/
int gp3ready(void)
{
  int s;
  usleep(100); // wait 100uS in case GP3 is just starting a command
  ioctl(fd,TIOCMGET,&s);
  return (s&TIOCM_CTS)==TIOCM_CTS;
}





